@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -89,6 +90,13 @@ Basic_buffer::Basic_buffer( const Basic_buffer & b, const Point & p1, const Poin
   }
 
 
+bool Basic_buffer::blank( const int line ) const throw()
+  {
+  Point p( line, 0 );
+  return ( !pisvalid( p ) || bot( p ) == eol( p ) );
+  }
+
+
 int Basic_buffer::characters( const int line ) const throw()
   {
   if( line < 0 || line >= lines() ) return -1;
@@ -163,7 +171,7 @@ bool Basic_buffer::pprev( Point & p ) const throw()
   }
 
 
-bool Basic_buffer::pseek( Point & p, int n ) const throw()
+bool Basic_buffer::pseek( Point & p, const int n ) const throw()
   {
   Point saved_p = p;
   p.col += n;
@@ -277,6 +285,26 @@ bool Basic_buffer::pdelb( const Point & p1, const Point & p2 )
   }
 
 
+bool Basic_buffer::capitalize( const Point & p1, const Point & p2 ) throw()
+  {
+  bool change = false;
+  bool prev_is_alnum = false;
+  if( p1 < p2 && pisvalid( p1 ) && pisvalid( p2 ) )
+    for( Point p = p1; p < p2; pnext( p ) )
+      {
+      const unsigned char ch = data[p.line][p.col];
+      if( prev_is_alnum )
+        { if( ISO_8859::isupper( ch ) )
+            { data[p.line][p.col] = ISO_8859::tolower( ch ); change = true; } }
+      else
+        { if( ISO_8859::islower( ch ) )
+            { data[p.line][p.col] = ISO_8859::toupper( ch ); change = true; } }
+      prev_is_alnum = ISO_8859::isalnum_( ch );
+      }
+  return change;
+  }
+
+
 bool Basic_buffer::to_lowercase( const Point & p1, const Point & p2 ) throw()
   {
   bool change = false;
