@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,11 +39,11 @@
 
 namespace {
 
-const char * invocation_name = 0;
-const char * const Program_name     = "GNU Moe";
-const char * const program_name     = "moe";
+const char * const     Program_name = "GNU Moe";
+const char * const     program_name = "moe";
 const char * const config_file_name = "moerc";
-const char * const program_year     = "2009";
+const char * const     program_year = "2011";
+const char * invocation_name = 0;
 
 
 void show_help() throw()
@@ -54,25 +55,25 @@ void show_help() throw()
   std::printf( "\nEditor options:\n" );
   std::printf( "  -h, --help               display this help and exit\n" );
   std::printf( "  -V, --version            output version information and exit\n" );
+  std::printf( "  -1, --orphan             put extra files in orphaned buffers\n" );
   std::printf( "  -b, --backup             create backup files (default)\n" );
-  std::printf( "  -e, --exit-ask           save-and-close always asks for confirmation\n" );
+  std::printf( "  -e, --exit-ask           make save-and-close ask always for confirmation\n" );
   std::printf( "  -H, --smart-home         go home, then go to first non-blank character\n" );
   std::printf( "  -i, --ignore-case        make search case insensitive by default\n" );
   std::printf( "  -k, --keep-lines=<n>     number of lines to keep for PgUp/PgDn\n" );
   std::printf( "  -m, --max-windows=<n>    max number of windows to show at once\n" );
   std::printf( "  -n, --indent-step=<n>    number of spaces to add when indenting a block\n" );
-  std::printf( "  -1, --orphan             put extra files in orphaned buffers\n" );
 //  std::printf( "  -p, --beep               beep on errors and when cursor goes past extremes\n" );
-  std::printf( "  -s, --search-wrap        search wraps at end of file\n" );
+  std::printf( "  -s, --search-wrap        make search wrap at end of file\n" );
   std::printf( "  -u, --auto-unmark        turn off highlighting after block copy or move\n" );
   std::printf( "  -x, --rectangle          rectangular block mode\n" );
   std::printf( "\nFile options:\n" );
-  std::printf( "  -a, --auto-indent        auto indent\n" );
-  std::printf( "  -l, --lmargin=<col>      left margin\n" );
-  std::printf( "  -r, --rmargin=<col>      right margin\n" );
-  std::printf( "  -o, --read-only          file is read-only\n" );
-  std::printf( "  -O, --overwrite          insert mode off\n" );
-  std::printf( "  -w, --word-wrap          word wrap\n" );
+  std::printf( "  -a, --auto-indent        enable auto indent\n" );
+  std::printf( "  -l, --lmargin=<col>      set left margin\n" );
+  std::printf( "  -r, --rmargin=<col>      set right margin\n" );
+  std::printf( "  -o, --read-only          make buffer read-only\n" );
+  std::printf( "  -O, --overwrite          disable insert mode\n" );
+  std::printf( "  -w, --word-wrap          enable word wrapping\n" );
   std::printf( "\nReport bugs to bug-moe@gnu.org\n" );
   std::printf( "Moe home page: http://www.gnu.org/software/moe/moe.html\n" );
   std::printf( "General help using GNU software: http://www.gnu.org/gethelp\n" );
@@ -181,21 +182,23 @@ const std::string & RC::home_directory() throw()
   }
 
 
-void RC::show_error( const char * msg, const int errcode, const bool help ) throw()
+void RC::show_error( const char * const msg, const int errcode,
+                     const bool help ) throw()
   {
-  if( msg && msg[0] != 0 )
+  if( msg && msg[0] )
     {
     std::fprintf( stderr, "%s: %s", program_name, msg );
     if( errcode > 0 )
       std::fprintf( stderr, ": %s", std::strerror( errcode ) );
     std::fprintf( stderr, "\n" );
     }
-  if( help && invocation_name && invocation_name[0] != 0 )
-    std::fprintf( stderr, "Try `%s --help' for more information.\n", invocation_name );
+  if( help && invocation_name && invocation_name[0] )
+    std::fprintf( stderr, "Try `%s --help' for more information.\n",
+                  invocation_name );
   }
 
 
-int main( const int argc, const char * argv[] )
+int main( const int argc, const char * const argv[] )
   {
   invocation_name = argv[0];
 
@@ -221,7 +224,7 @@ int main( const int argc, const char * argv[] )
     }
   if( retval ) return retval;
 
-  if( !Screen::init() ) { RC::show_error( "screen too small" ); return 1; }
+  if( !Screen::init() ) { RC::show_error( "Screen too small." ); return 1; }
   set_signals();
 
   char buf[88];
