@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,8 +32,8 @@
 #include "window.h"
 
 
-Window::Window( const int tl, const int h, Buffer_handle & bh, const bool center ) throw()
-  : _top_line( tl ), _height( h ), bhp( &bh )
+Window::Window( Buffer_handle & bh, const int tl, const int h, const bool center ) throw()
+  : bhp( &bh ), top_line_( tl ), height_( h )
   {
   bhp->buffer().pvalid( bhp->pointer );
   if( center ) center_cursor();
@@ -41,11 +42,11 @@ Window::Window( const int tl, const int h, Buffer_handle & bh, const bool center
 
 
 Point Window::clock_position() const throw()
-  { return Point( _top_line, Screen::width() - 20 ); }
+  { return Point( top_line_, Screen::width() - 20 ); }
 
 
 Point Window::absolute_cursor() const throw()	// cursor position on screen
-  { return relative_cursor() + Point( _top_line, 0 ); }
+  { return relative_cursor() + Point( top_line_, 0 ); }
 
 
 Point Window::relative_cursor() const throw()	// cursor position on window
@@ -101,6 +102,8 @@ void Window::goto_begin_of_block() throw()
   {
   if( Block::bufferp() == &buffer() && buffer().pisvalid( Block::begin() ) )
     update_points( Block::begin(), true, true );
+  else if( !Block::bufferp() || Block::begin().line < 0 )
+    Screen::show_message( "Begin of block is not set" );
   else Screen::show_message( "Begin of block is not in this file" );
   }
 
@@ -109,6 +112,8 @@ void Window::goto_end_of_block() throw()
   {
   if( Block::bufferp() == &buffer() && buffer().pisvalid( Block::end() ) )
     update_points( Block::end(), true, true );
+  else if( !Block::bufferp() || Block::end().line < 0 )
+    Screen::show_message( "End of block is not set" );
   else Screen::show_message( "End of block is not in this file" );
   }
 
@@ -239,10 +244,10 @@ void Window::goto_pprev() throw()
 void Window::move_page( const bool down ) throw()
   {
   const int keep_lines = RC::editor_options().keep_lines;
-  int lines = ( _height - 1 ) / 2;
+  int lines = ( height_ - 1 ) / 2;
 
   if( keep_lines >= 0 && keep_lines <= lines )
-    lines = _height - 1 - keep_lines;
+    lines = height_ - 1 - keep_lines;
   if( !down ) lines = -lines;
   move_vertical( lines );
   }
@@ -281,10 +286,10 @@ void Window::scroll_horizontal( const int cols ) throw()
 void Window::scroll_page( const bool down ) throw()
   {
   const int keep_lines = RC::editor_options().keep_lines;
-  int lines = ( _height - 1 ) / 2;
+  int lines = ( height_ - 1 ) / 2;
 
   if( keep_lines >= 0 && keep_lines < lines )
-    lines = _height - 1 - keep_lines;
+    lines = height_ - 1 - keep_lines;
   if( !down ) lines = -lines;
   scroll_vertical( lines );
   }
@@ -297,7 +302,7 @@ void Window::scroll_vertical( const int lines ) throw()
   if( lines < 0 ) { if( tl.line == 0 ) { return; } tl.line += lines; tl.cut(); }
   else if( lines > 0 )
     {
-    int first_line = bhp->buffer().last_line() - ( _height - 2 );
+    int first_line = bhp->buffer().last_line() - ( height_ - 2 );
     if( first_line < 0 || tl.line == first_line ) return;
     tl.line += lines; if( tl.line > first_line ) tl.line = first_line;
     }
@@ -322,33 +327,34 @@ void Window::set_mark( const int i ) throw()
 void Window::repaint() const throw()
   {
   show_status_line();
-  for( int i = 0; i < _height - 1; ++i )
-    {
+  for( int i = 0; i < height_ - 1; ++i )
     Screen::out_buffer_line( bhp->buffer(), bhp->top_left + Point( i, 0 ),
-                             _top_line + i + 1 );
-    }
+                             top_line_ + i + 1 );
   }
 
 
 void Window::show_character_info() const throw()
   {
-  Screen::show_message( bhp->buffer().character_info( bhp->pointer ) );
+  std::string s;
+  bhp->buffer().character_info( bhp->pointer, s );
+  Screen::show_message( s );
   }
 
 
 // prefix == keyboard status ( ^K, ^Q, ^[, etc )
 //
-void Window::show_status_line( const char * prefix ) const throw()
+void Window::show_status_line( const char * const prefix ) const throw()
   {
-  if( _top_line < 0 ) return;
-  char buf[Screen::width()+1];
-  int offset = Screen::width() - 40;
+  if( top_line_ < 0 ) return;
+  int size = 0;
+  char * const buf = Screen::line_buf( &size );
+  int offset = size - 41;
   const int line = bhp->pcursor.line + 1, col = bhp->pcursor.col + 1;
   for( int i = line; i > 9999; i /= 10 ) --offset;
   for( int i = col; i > 999; i /= 10 ) --offset;
-  const char * uname = buffer().uname().c_str();
-  const int overlap = buffer().uname().size() + 11 - offset;
-  if( overlap > 0 ) uname += overlap;
+  const char * name = buffer().nename().c_str();
+  const int overlap = buffer().nename().size() + 11 - offset;
+  if( overlap > 0 ) name += overlap;
   const bool modified = buffer().modified();
   const int len = snprintf( buf, offset + 1, "%-3s %c%c%c%c%c%c %s%s",
                             prefix ? prefix : "   ",
@@ -357,22 +363,22 @@ void Window::show_status_line( const char * prefix ) const throw()
                             buffer().options.auto_indent ? 'A' : ' ',
                             buffer().options.read_only ? 'R' : ' ',
                             RC::editor_options().rectangle_mode ? 'X' : ' ',
-                            modified ? '*' : ' ', uname,
+                            modified ? '*' : ' ', name,
                             modified ? " (Modified)" : "" );
   if( len < offset ) std::memset( buf + len, ' ', offset - len );
-  snprintf( buf + offset, sizeof( buf ) - offset,
+  snprintf( buf + offset, size - offset,
             "  Line %-4d Col %-3d %5s  F1 for help",
-            line, col, Screen::clock_string().c_str() );
+            line, col, Screen::clock_string() );
   if( RC::editor_options().show_code )
     {
-    offset = Screen::width() - 13;
+    offset = size - 14;
     const int ch = bhp->buffer()[bhp->pointer];
     if( ch >= 0 )
-      snprintf( buf + offset, sizeof( buf ) - offset, "%3d(0x%02X)", ch, ch );
+      snprintf( buf + offset, size - offset, "%3d(0x%02X)", ch, ch );
     else
-      snprintf( buf + offset, sizeof( buf ) - offset, "End of file" );
+      snprintf( buf + offset, size - offset, "End of file" );
     }
-  Screen::out_line( buf, _top_line, false, 'i' );
+  Screen::out_line( buf, top_line_, false, 'i' );
   }
 
 
@@ -385,31 +391,34 @@ void Window::update_points( const Point & to, const bool show,
   Point & c = bh.cursor;
   Point & p = bh.pointer;
   Point & pc = bh.pcursor;
-  Point old_tl = tl;
+  const Point old_tl = tl;
 
   switch( from )
     {
     case from_top_left:
-      { Point dif = c - tl;
+      {
+      const Point dif = c - tl;
       tl = to; tl.cut(); c = tl + dif; p = buffer.to_pointer( c, pc );
-      center = false; break; }
+      center = false;
+      break;
+      }
     case from_cursor: c = to; c.cut(), p = buffer.to_pointer( c, pc ); break;
     case from_pointer: p = to; c = pc = buffer.to_cursor( p ); break;
     }
 
   const Point rc = relative_cursor();
-  if( rc.line > _height - 2 )
+  if( rc.line > height_ - 2 )
     {
-    const int d = center ? ( _height - 2 ) / 2 : _height - 2;
+    const int d = center ? ( height_ - 2 ) / 2 : height_ - 2;
     tl.line = std::max( 0, c.line - d );
     }
   else if( rc.line < 1 )
     {
-    const int d = center ? ( _height - 2 ) / 2 : 0;
+    const int d = center ? ( height_ - 2 ) / 2 : 0;
     tl.line = std::max( 0, c.line - d );
     }
-  if( rc.col >= Screen::width() ) tl.col = c.col - Screen::width() + 1;
-  else if( rc.col < 0 ) tl.col = std::max( 0, c.col - ( Screen::width() / 2 ) );
+  if( rc.col >= Screen::width() || rc.col <= 0 || center )
+    tl.col = std::max( 0, c.col - Screen::width() + 1 );
 
   if( show ) { if( tl == old_tl ) show_status_line(); else repaint(); }
   }
