@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +18,7 @@
 
 #include <algorithm>
 #include <cctype>
+#include <cstdio>
 #include <ctime>
 #include <string>
 #include <vector>
@@ -36,7 +38,7 @@
 
 namespace Window_vector {
 
-int _curwin;			// current window (the one with the cursor in)
+int curwin_;			// current window (the one with the cursor in)
 std::vector< Window > data;
 
 
@@ -90,7 +92,7 @@ void replace_and_repaint( Buffer & buffer, const Basic_buffer & tmp ) throw()
 
 int Window_vector::init()
   {
-  _curwin = 0;
+  curwin_ = 0;
   return load( RC::editor_options().orphan_extra ? 1 : Bufhandle_vector::handles() );
   }
 
@@ -104,30 +106,30 @@ int Window_vector::load( int new_size )
   const bool toggle = ( new_size < 0 && handles > 1 );
   const bool center = ( data.size() == 0 );		// center all on init
 
-  if( !handles ) { _curwin = 0; data.clear(); return 0; }
+  if( !handles ) { curwin_ = 0; data.clear(); return 0; }
   if( new_size == 0 ) new_size = data.size();
   else if( new_size < 0 )
     {
-    for( int i = 0; i < _curwin; ++i ) Bufhandle_vector::next();
+    for( int i = 0; i < curwin_; ++i ) Bufhandle_vector::next();
     new_size = (data.size() != 1) ? 1 : handles;
-    _curwin = 0;
+    curwin_ = 0;
     }
   new_size = std::min( new_size, handles );
   new_size = std::min( new_size, Screen::max_windows() );
   if( RC::editor_options().max_windows > 0 )
     new_size = std::min( new_size, RC::editor_options().max_windows );
   new_size = std::max( new_size, 1 );
-  while( _curwin >= new_size ) { --_curwin; Bufhandle_vector::next(); }
+  while( curwin_ >= new_size ) { --curwin_; Bufhandle_vector::next(); }
   const int wheight = Screen::height() / new_size;
 
   data.clear();
   int i = 0, y = 0;
   while( i + 1 < new_size )
     {
-    data.push_back( Window( y, wheight, Bufhandle_vector::handle( i ), center ) );
+    data.push_back( Window( Bufhandle_vector::handle( i ), y, wheight, center ) );
     ++i; y += wheight;
     }
-  data.push_back( Window( y, Screen::height() - y, Bufhandle_vector::handle( i ), center ) );
+  data.push_back( Window( Bufhandle_vector::handle( i ), y, Screen::height() - y, center ) );
   if( toggle && data.size() == 1 ) data[0].center_cursor();
   for( i = 0; i < (int)data.size(); ++i ) data[i].repaint();
   return data.size();
@@ -136,14 +138,14 @@ int Window_vector::load( int new_size )
 
 void Window_vector::bufhandle_menu( const int abort_key )
   {
-  const int handle = Menu::bufhandle_menu( _curwin, abort_key );
-  if( handle >= 0 ) { _curwin = handle; if( _curwin >= windows() ) load(); }
+  const int handle = Menu::bufhandle_menu( curwin_, abort_key );
+  if( handle >= 0 ) { curwin_ = handle; if( curwin_ >= windows() ) load(); }
   }
 
 
 void Window_vector::change_buffer_name()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
 
   std::string name( buffer.name() );
   if( Menu::get_filename( "New buffer name (^C to abort): ", name ) < 0 )
@@ -161,9 +163,9 @@ void Window_vector::load_file( const int abort_key )
   const int old_handles = Bufhandle_vector::handles();
   int handle;
   try { handle = Bufhandle_vector::find_or_add_handle( RC::default_buffer_options(),
-                                                       &name, _curwin + 1 ); }
-  catch( Buffer::Error e ) { Screen::show_message( e.s ); return; }
-  _curwin = handle;
+                                                       &name, curwin_ + 1 ); }
+  catch( Buffer::Error e ) { Screen::show_message( e.msg ); return; }
+  curwin_ = handle;
   if( windows() > 1 && Bufhandle_vector::handles() > old_handles )
     load( windows() + 1 );
   else load();
@@ -172,21 +174,21 @@ void Window_vector::load_file( const int abort_key )
 
 void Window_vector::split_window()
   {
-  Bufhandle_vector::duplicate_handle( _curwin++ );
+  Bufhandle_vector::duplicate_handle( curwin_++ );
   load( windows() + 1 );
   }
 
 
 void Window_vector::save_file( const int abort_key )
   {
-  const Buffer & buffer = data[_curwin].buffer();
+  const Buffer & buffer = data[curwin_].buffer();
 
   std::string name( buffer.name() );
   if( Menu::get_filename( "Name of file to save (^C to abort): ", name, abort_key ) <= 0 )
     return;
   bool done;
   try { done = buffer.save( &name ); }
-  catch( Buffer::Error e ) { Screen::show_message( e.s ); return; }
+  catch( Buffer::Error e ) { Screen::show_message( e.msg ); return; }
   if( done )
     {
     Screen::show_status_lines( &buffer );
@@ -201,7 +203,7 @@ void Window_vector::save_file( const int abort_key )
 //
 int Window_vector::close( bool abort, const bool quiet )
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
   Buffer_handle & bh = w.buffer_handle();
   const bool last_handle = Bufhandle_vector::handles( bh.buffer() ) <= 1;
   const bool modified = bh.buffer().modified();
@@ -217,13 +219,14 @@ int Window_vector::close( bool abort, const bool quiet )
       }
     if( !abort && modified )
       {
+      if( read_only( bh.buffer() ) ) return -1;
       std::string name( bh.buffer().name() );
       if( exit_ask || !name.size() )
         while( !Menu::get_filename( "Name of file to save (^C to abort): ", name ) ) ;
       if( !name.size() ) return -1;
       try { if( !bh.buffer().save( &name ) ) return -1; }
       catch( Buffer::Error e )
-        { if( !quiet ) Screen::show_message( e.s ); return -1; }
+        { if( !quiet ) Screen::show_message( e.msg ); return -1; }
       }
     else if( exit_ask && last_handle && !modified )
       {
@@ -231,8 +234,8 @@ int Window_vector::close( bool abort, const bool quiet )
       if( key != 'y' && key != 'Y' ) return -1;
       }
     }
-  Bufhandle_vector::delete_handle( _curwin );
-  if( _curwin > 0 ) --_curwin;
+  Bufhandle_vector::delete_handle( curwin_ );
+  if( curwin_ > 0 ) --curwin_;
   return load();
   }
 
@@ -251,15 +254,15 @@ int Window_vector::close_and_exit( const bool abort )
   }
 
 
-Window & Window_vector::curwin() throw() { return data[_curwin]; }
+Window & Window_vector::curwin() throw() { return data[curwin_]; }
 
 
 void Window_vector::last_visited()
   {
   const int handles = Bufhandle_vector::handles();
   if( handles <= 1 ) return;
-  _curwin = Bufhandle_vector::last_visited( _curwin );
-  if( _curwin >= windows() ) load();
+  curwin_ = Bufhandle_vector::last_visited( curwin_ );
+  if( curwin_ >= windows() ) load();
   }
 
 
@@ -267,10 +270,10 @@ void Window_vector::next()
   {
   const int handles = Bufhandle_vector::handles();
   if( handles <= 1 ) return;
-  if( _curwin + 1 < windows() ) ++_curwin;
+  if( curwin_ + 1 < windows() ) ++curwin_;
   else
     {
-    if( handles <= windows() ) _curwin = 0;
+    if( handles <= windows() ) curwin_ = 0;
     else { Bufhandle_vector::next(); load(); }
     }
   }
@@ -280,10 +283,10 @@ void Window_vector::prev()
   {
   const int handles = Bufhandle_vector::handles();
   if( handles <= 1 ) return;
-  if( _curwin > 0 ) --_curwin;
+  if( curwin_ > 0 ) --curwin_;
   else
     {
-    if( handles <= windows() ) _curwin = windows() - 1;
+    if( handles <= windows() ) curwin_ = windows() - 1;
     else { Bufhandle_vector::prev(); load(); }
     }
   }
@@ -299,26 +302,32 @@ Window & Window_vector::window( int i ) throw()
 int Window_vector::windows() throw() { return data.size(); }
 
 
-void Window_vector::add_char( unsigned char ch, const bool force ) throw()
+void Window_vector::add_char( const int key, const bool force ) throw()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
   if( w.buffer().options.read_only )
     {
-    if( ch == ' ' && !force ) w.move_vertical( w.height() - 2 );
+    if( !force && key == ' ' ) w.move_vertical( w.height() - 2 );
     else read_only( w.buffer() );
     return;
     }
 
-  if( ch == '\r' && !force ) ch = '\n';
-  if( ( force || ch == '\t' || ch == '\n' || ISO_8859::isprint( ch ) ) &&
-      w.buffer().pputc( w.pointer(), ch ) )
-    Screen::repaint( &w.buffer() );
+  if( key >= 0 && key < 256 &&
+      ( force || key == '\t' || key == '\n' || ISO_8859::isprint( key ) ) )
+    { if( w.buffer().pputc( w.pointer(), key ) )
+        Screen::repaint( &w.buffer() ); }
+  else
+    {
+    char buf[64];
+    snprintf( buf, sizeof buf, "Key code %d(0x%02X) ignored", key, key );
+    Screen::show_message( buf );
+    }
   }
 
 
 void Window_vector::delete_char( const bool back ) throw()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
   Point & p = w.pointer();
 
   if( !read_only( w.buffer() ) && w.buffer().pdelc( p, back ) )
@@ -328,7 +337,7 @@ void Window_vector::delete_char( const bool back ) throw()
 
 void Window_vector::delete_line() throw()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
 
   if( !read_only( w.buffer() ) && w.buffer().pdell( w.pointer() ) )
     Screen::repaint( &w.buffer() );
@@ -337,7 +346,7 @@ void Window_vector::delete_line() throw()
 
 void Window_vector::copy_block() throw()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
 
   if( !no_block() && !read_only( w.buffer() ) )
     {
@@ -353,7 +362,7 @@ void Window_vector::copy_block() throw()
 
 void Window_vector::delete_block() throw()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
 
   if( !no_block() && !no_block_in_this_file( buffer ) &&
       !read_only( buffer ) && Block::delete_block() )
@@ -363,7 +372,7 @@ void Window_vector::delete_block() throw()
 
 void Window_vector::move_block() throw()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
   Point & p = w.pointer();
 
   if( !no_block() && !Block::in_block_or_end( w.buffer(), p ) )
@@ -381,7 +390,7 @@ void Window_vector::move_block() throw()
 
 void Window_vector::read_block()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
   if( read_only( w.buffer() ) ) return;
 
   std::string name;
@@ -394,7 +403,7 @@ void Window_vector::read_block()
     if( w.buffer().pputb( p, tmp, tmp.bof(), tmp.eof() ) )
       Screen::repaint( &w.buffer() );
     }
-  catch( Buffer::Error e ) { Screen::show_message( e.s ); }
+  catch( Buffer::Error e ) { Screen::show_message( e.msg ); }
   }
 
 
@@ -418,17 +427,17 @@ void Window_vector::write_block()
       }
     if( RC::editor_options().auto_unmark )
       {
-      const Buffer * bufferp = Block::bufferp();
+      const Buffer * const bufferp = Block::bufferp();
       Block::reset(); Screen::repaint( bufferp );
       }
     }
-  catch( Buffer::Error e ) { Screen::show_message( e.s ); }
+  catch( Buffer::Error e ) { Screen::show_message( e.msg ); }
   }
 
 
 void Window_vector::indent_block() throw()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
 
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
       read_only( buffer ) ) return;
@@ -457,7 +466,7 @@ void Window_vector::indent_block() throw()
 
 void Window_vector::unindent_block() throw()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
 
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
       read_only( buffer ) ) return;
@@ -508,7 +517,7 @@ void Window_vector::unindent_block() throw()
 
 void Window_vector::undo() throw()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
 
   if( !read_only( w.buffer() ) )
     {
@@ -520,7 +529,7 @@ void Window_vector::undo() throw()
 
 void Window_vector::redo() throw()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
 
   if( !read_only( w.buffer() ) )
     {
@@ -532,7 +541,7 @@ void Window_vector::redo() throw()
 
 void Window_vector::center_line() throw()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
   Buffer & buffer = w.buffer();
   const Point bol = buffer.bol( w.pointer() );
   const Point bot = buffer.bot( bol );
@@ -560,7 +569,7 @@ void Window_vector::center_line() throw()
 
 void Window_vector::reformat() throw()
   {
-  Window & w = data[_curwin];
+  Window & w = data[curwin_];
   Buffer & buffer = w.buffer();
   const Point bop = buffer.bop( w.pointer() );
   const Point eop = buffer.eop( w.pointer() );
@@ -607,7 +616,7 @@ void Window_vector::reformat() throw()
 
 void Window_vector::change_case( const bool to_caps ) throw()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || read_only( buffer ) )
     return;
   Basic_buffer tmp;
@@ -638,7 +647,7 @@ void Window_vector::change_case( const bool to_caps ) throw()
 
 void Window_vector::encode_base64() throw()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
       read_only( buffer ) ) return;
   std::string in, out;
@@ -659,7 +668,7 @@ void Window_vector::encode_base64() throw()
 
 void Window_vector::decode_base64() throw()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
       read_only( buffer ) ) return;
   std::string in, out;
@@ -679,7 +688,7 @@ void Window_vector::decode_base64() throw()
 
 void Window_vector::decode_quoted_printable_utf8( const bool mode_q ) throw()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
       read_only( buffer ) ) return;
   std::string in, out;
@@ -702,7 +711,7 @@ void Window_vector::decode_quoted_printable_utf8( const bool mode_q ) throw()
 
 void Window_vector::encode_rot1347( const bool mode13 ) throw()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
       read_only( buffer ) ) return;
   Basic_buffer tmp;
@@ -719,7 +728,7 @@ void Window_vector::encode_rot1347( const bool mode13 ) throw()
 
 void Window_vector::remove_duplicate_lines( const bool back ) throw()
   {
-  Buffer & buffer = data[_curwin].buffer();
+  Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
       read_only( buffer ) ) return;
 
@@ -754,8 +763,8 @@ void Window_vector::remove_duplicate_lines( const bool back ) throw()
     buffer.reset_appendable();
     if( Block::valid() )
       {
-      if( back ) data[_curwin].update_points( Block::end(), true, true );
-      else data[_curwin].update_points( Block::begin(), true, true );
+      if( back ) data[curwin_].update_points( Block::end(), true, true );
+      else data[curwin_].update_points( Block::begin(), true, true );
       }
     if( RC::editor_options().auto_unmark ) Block::reset();
     Screen::repaint( &buffer );
@@ -764,13 +773,14 @@ void Window_vector::remove_duplicate_lines( const bool back ) throw()
   }
 
 
-void Window_vector::copyright_update()
+const char * Window_vector::copyright_update()
   {
   static std::vector< std::string > history;
   const std::string search_string( "\\<Copyright\\>\\+\\[^\\n]\\+\\+\\+\\[0-9]\\*\\<\\[^0-9]" );
 
   std::string year_string;
-  time_t dummy = time( 0 ); struct tm *t = localtime( &dummy );
+  const time_t tt = std::time( 0 );
+  const std::tm * const t = std::localtime( &tt );
   if( t )
     {
     int year = 1900 + t->tm_year;
@@ -779,18 +789,17 @@ void Window_vector::copyright_update()
     }
   history.push_back( year_string );
   if( Screen::get_string( "Year to add (^C to abort): ", history ) <= 0 )
-    return;
+    return 0;
   year_string = history.back();
   for( unsigned int i = 0; i < year_string.size(); ++i )
     if( !std::isdigit( year_string[i] ) )
       year_string.clear();
-  if( !year_string.size() || year_string[0] == '0' )
-    { Screen::show_message( "Invalid year" ); return; }
+  if( !year_string.size() || year_string[0] == '0' ) return "Invalid year";
   year_string.insert( 0, "\\<" ); year_string += "\\>";
   bool ask = true, found = false, modified = false;
   for( int i = 0; i < Bufhandle_vector::handles(); next(), ++i )
     {
-    Window & w = data[_curwin];
+    Window & w = data[curwin_];
     if( w.buffer().options.read_only ) continue;
     Point p = w.buffer().bof();
     std::vector< std::string > regex_pieces;
@@ -802,35 +811,35 @@ void Window_vector::copyright_update()
       found = true;
       const Point p0 = w.buffer().bol( p1 );
       const Point p2 = w.buffer().eol( p );
+      w.buffer().pprev( p );
+      const Point ph = p;		// copyright holder begins here
       while( w.buffer().pprev( p ) && !w.buffer().piseow( p ) ) ;
       Basic_buffer tmp( w.buffer(), p0, p );
-      Point pp = tmp.bof();
+      Point pt = tmp.bof();
       if( p1 < p && w.buffer().piseow( p ) &&
-          !Regex::find( tmp, pp, pp, year_string, regex_pieces ) )
+          !Regex::find( tmp, pt, pt, year_string, regex_pieces ) )
         {
         const int rmargin = w.buffer().options.rmargin();
-        pp = tmp.eof();
-        tmp.pputc( pp, ',' );
-        if( tmp.to_cursor( pp ).col + (int)year_string.size() - 4 > rmargin + 1 )
+        pt = tmp.eof();
+        tmp.pputc( pt, ',' );
+        if( tmp.to_cursor( pt ).col + (int)year_string.size() - 4 > rmargin + 1 )
           {
-          tmp.pputc( pp, '\n' );
-          tmp.pputb( pp, w.buffer(), p0, p1 );
+          tmp.pputc( pt, '\n' );
+          tmp.pputb( pt, w.buffer(), p0, p1 );
           }
-        else tmp.pputc( pp, ' ' );
+        else tmp.pputc( pt, ' ' );
         for( unsigned int i = 2; i < year_string.size() - 2; ++i )
-          tmp.pputc( pp, year_string[i] );
-        while( p < p2 && std::ispunct( w.buffer()[p] ) && w.buffer().pnext( p ) ) ;
-        while( p < p2 && std::isspace( w.buffer()[p] ) && w.buffer().pnext( p ) ) ;
-        if( p < p2 )
+          tmp.pputc( pt, year_string[i] );
+        if( ph < p2 )
           {
-          Point pp1 = pp;
-          tmp.pputb( pp, w.buffer(), p, p2 );
-          if( tmp.to_cursor( pp ).col + 1 > rmargin + 1 )
+          Point pt1 = pt;
+          tmp.pputb( pt, w.buffer(), ph, p2 );
+          if( tmp.to_cursor( pt ).col + 1 > rmargin + 1 )
             {
-            tmp.pputc( pp1, '\n' );
-            tmp.pputb( pp1, w.buffer(), p0, p1 );
+            tmp.pputc( pt1, '\n' );
+            tmp.pputb( pt1, w.buffer(), p0, p1 );
             }
-          else tmp.pputc( pp1, ' ' );
+          else tmp.pputc( pt1, ' ' );
           }
         p = p2;
         w.update_points( p, true, true );
@@ -838,7 +847,7 @@ void Window_vector::copyright_update()
         if( ask || Screen::kbhit() >= 0 )
           {
           key = Screen::show_message( "Update (Y)es (N)o (R)est (^C to abort): ", true );
-          if( key == 3 ) return;
+          if( key == 3 ) return 0;
           if( ask && ( key == 'r' || key == 'R' ) ) ask = false;
           }
         if( !ask || key == 'y' || key == 'Y' )
@@ -851,17 +860,16 @@ void Window_vector::copyright_update()
         }
       }
     }
-  if( !found ) Screen::show_message( "No copyright notices found" );
-  else if( !modified )
-    Screen::show_message( "No copyright notices have been modified" );
-  else Screen::show_message( "Finished" );
+  if( !found ) return "No copyright notices found";
+  if( !modified ) return "No copyright notices have been modified";
+  return "Finished";
   }
 
 
-void Window_vector::search_word( const int abort_key )
+const char * Window_vector::search_word( const int abort_key )
   {
-  const Buffer & buffer = data[_curwin].buffer();
-  Point p1 = data[_curwin].pointer();
+  const Buffer & buffer = data[curwin_].buffer();
+  Point p1 = data[curwin_].pointer();
   if( ISO_8859::isalnum_( buffer[p1] ) || buffer.piseow( p1 ) )
     {
     Point p2 = p1;
@@ -872,14 +880,16 @@ void Window_vector::search_word( const int abort_key )
       std::string s_ini;
       buffer.to_string( p1, p2, s_ini );
       s_ini.insert( 0, "\\<" ); s_ini += "\\>";
-      search( abort_key, false, s_ini );
+      return search( abort_key, false, s_ini );
       }
     }
+  return 0;
   }
 
 
-void Window_vector::search( const int abort_key, const bool again,
-                            const std::string & s_ini )
+const char * Window_vector::search( const int abort_key, const bool again,
+                                    const std::string & s_ini,
+                                    const bool toggle )
   {
   static std::vector< std::string > search_history;
   static std::vector< std::string > replace_history;
@@ -889,22 +899,27 @@ void Window_vector::search( const int abort_key, const bool again,
   static bool backward = false, block = false, global = false, icase = false;
   static bool replace = false;
 
-  if( !again || search_history.size() == 0 )
+  if( !again || !search_string.size() )
     {
     search_history.push_back( s_ini );
-    if( Screen::get_string( "Find (^C to abort): ", search_history, abort_key, true ) <= 0 )
-      return;
+    int size = Screen::get_string( "Find (^C to abort): ", search_history,
+                                   abort_key );
+    if( size <= 0 )
+      {
+      if( s_ini.size() && search_history.size() ) search_history.pop_back();
+      return 0;
+      }
     search_string = search_history.back(); ISO_8859::deescapize( search_string );
     std::string prompt;
     if( RC::editor_options().ignore_case )
-      prompt = "(N)o_ignore (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
-    else prompt = "(I)gnore (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
+      prompt = "(N)o_ignore_case (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
+    else prompt = "(I)gnore_case (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
     options_history.push_back( std::string() );
-    const int result = Screen::get_string( prompt, options_history, abort_key );
-    if( result < 0 ) return;
+    size = Screen::get_string( prompt, options_history, abort_key );
+    if( size < 0 ) return 0;
     backward = false; block = false; global = false; replace = false;
     icase = RC::editor_options().ignore_case;
-    if( result > 0 )
+    if( size > 0 )
       {
       const std::string & options = options_history.back();
       for( unsigned int i = 0; i < options.size(); ++i )
@@ -922,18 +937,20 @@ void Window_vector::search( const int abort_key, const bool again,
       {
       replace_string.clear();
       replace_history.push_back( std::string() );
-      const int len = Screen::get_string( "Replace with (^C to abort): ", replace_history, abort_key, true );
-      if( len < 0 ) { replace = false; return; }
+      const int len = Screen::get_string( "Replace with (^C to abort): ",
+                                          replace_history, abort_key );
+      if( len < 0 ) { replace = false; return 0; }
       if( len > 0 )
         { replace_string = replace_history.back();
           ISO_8859::deescapize( replace_string ); }
       }
     }
+  if( toggle ) backward = !backward;
 
   Point p = curwin().pointer();
   if( block )
     {
-    if( no_block() || no_block_in_this_file( curwin().buffer() ) ) return;
+    if( no_block() || no_block_in_this_file( curwin().buffer() ) ) return 0;
     if( !again || !Block::in_block_or_end( curwin().buffer(), p ) )
       { if( backward ) p = Block::end(); else p = Block::begin(); }
     }
@@ -941,7 +958,7 @@ void Window_vector::search( const int abort_key, const bool again,
   bool empty_replace = false;
   for( int i = (global ? Bufhandle_vector::handles() - 1 : 0); i >= 0 && !found; --i )
     {
-    Window & w = data[_curwin];
+    Window & w = data[curwin_];
     std::vector< std::string > regex_pieces;
     if( replace && empty_replace && !backward ) w.buffer().pnext( p );
     Point p1 = p, p2 = p;
@@ -959,49 +976,46 @@ void Window_vector::search( const int abort_key, const bool again,
           ( p1.col < Block::begin().col || p2.col > Block::end().col ) )
         { ++i; found = false; continue; }
       found_any = true;
-      if( !replace ) w.update_points( p, true, true );
-      else
+      if( !replace ) { w.update_points( p, true, true ); break; }
+      std::string error;
+      Block::save_block_position(); w.update_points( p, false, true );
+      if( ask )
         {
-        std::string error;
-        Block::save_block_position(); w.update_points( p, false, true );
-        if( ask )
-          {
-          const bool rm = RC::editor_options().rectangle_mode;
-          RC::editor_options().rectangle_mode = false;
-          Block::set_block( w.buffer(), p1, p2 ); w.repaint();
-          RC::editor_options().rectangle_mode = rm;
-          }
-        while( true )
-          {
-          int key = 0;
-          if( ask || Screen::kbhit() >= 0 )
-            {
-            key = Screen::show_message( "Replace (Y)es (N)o (R)est (^C to abort): ", true );
-            if( key == 3 ) break;
-            if( ask )
-              { if( key == 'n' || key == 'N' ) { found = false; break; }
-                if( key == 'r' || key == 'R' ) ask = false; }
-            }
-          if( !ask || key == 'y' || key == 'Y' )
-            {
-            Block::restore_block_position();
-            if( ask ) w.buffer().reset_appendable();
-            const bool b = Regex::replace( w.buffer(), p1, p2, replace_string, regex_pieces );
-            if( ask ) w.buffer().reset_appendable();
-            Block::save_block_position();
-            if( b )
-              { if( empty_replace && ( p1 < p2 ) ) empty_replace = false;
-                found = false; if( backward ) p = p1; else p = p2;
-                w.update_points( p, false, true ); }
-            else if( w.buffer().options.read_only ) error = "Read only";
-            else error = "Replace error: check syntax";
-            break;
-            }
-          }
-        ++i; Block::restore_block_position();
-        if( ask || found ) Screen::repaint( &w.buffer() );
-        if( error.size() ) Screen::show_message( error );
+        const bool rm = RC::editor_options().rectangle_mode;
+        RC::editor_options().rectangle_mode = false;
+        Block::set_block( w.buffer(), p1, p2 ); w.repaint();
+        RC::editor_options().rectangle_mode = rm;
         }
+      while( true )
+        {
+        int key = 0;
+        if( ask || Screen::kbhit() >= 0 )
+          {
+          key = Screen::show_message( "Replace (Y)es (N)o (R)est (^C to abort): ", true );
+          if( key == 3 ) break;
+          if( ask )
+            { if( key == 'n' || key == 'N' ) { found = false; break; }
+              if( key == 'r' || key == 'R' ) ask = false; }
+          }
+        if( !ask || key == 'y' || key == 'Y' )
+          {
+          Block::restore_block_position();
+          if( ask ) w.buffer().reset_appendable();
+          const bool b = Regex::replace( w.buffer(), p1, p2, replace_string, regex_pieces );
+          if( ask ) w.buffer().reset_appendable();
+          Block::save_block_position();
+          if( b )
+            { if( empty_replace && ( p1 < p2 ) ) empty_replace = false;
+              found = false; if( backward ) p = p1; else p = p2;
+              w.update_points( p, false, true ); }
+          else if( w.buffer().options.read_only ) error = "Read only";
+          else error = "Replace error: check syntax";
+          break;
+          }
+        }
+      ++i; Block::restore_block_position();
+      if( ask || found ) Screen::repaint( &w.buffer() );
+      if( error.size() ) Screen::show_message( error );
       continue;
       }
     if( !ask ) w.repaint();
@@ -1012,10 +1026,11 @@ void Window_vector::search( const int abort_key, const bool again,
         {
         if( backward ) { prev(); p = curwin().buffer().eof(); }
         else { next(); p = curwin().buffer().bof(); }
+        if( i == 0 ) { wrapped = true; ++i; }
         }
-      if( ( global && i == 0 ) || ( !global && RC::editor_options().search_wrap ) )
+      else if( RC::editor_options().search_wrap )
         {
-        if( backward ) p = curwin().buffer().eof();
+        if( backward ) p = w.buffer().eof();
         else p = w.buffer().bof();
         wrapped = true; ++i;
         }
@@ -1023,8 +1038,9 @@ void Window_vector::search( const int abort_key, const bool again,
     }
   if( !found_any )
     {
-    if( block ) Screen::show_message( "Not found (search restricted to marked block)" );
-    else Screen::show_message( "Not found" );
+    if( block ) return "Not found (search restricted to marked block)";
+    else return "Not found";
     }
-  else if( wrapped ) Screen::show_message( "Wrapped" );
+  else if( wrapped ) return "Wrapped";
+  return 0;
   }
