@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,7 +36,7 @@ void prev();
 Window & window( int i ) throw();
 int windows() throw();
 
-void add_char( unsigned char ch, const bool force = false ) throw();
+void add_char( const int key, const bool force = false ) throw();
 void delete_char( const bool back = false ) throw();
 void delete_line() throw();
 void copy_block() throw();
@@ -57,9 +58,10 @@ void decode_quoted_printable_utf8( const bool mode_q ) throw();
 void encode_rot1347( const bool mode13 ) throw();
 void remove_duplicate_lines( const bool back = false ) throw();
 
-void copyright_update();
-void search_word( const int abort_key );
-void search( const int abort_key, const bool again = false,
-             const std::string & s_ini = "" );
+const char * copyright_update();
+const char * search_word( const int abort_key );
+const char * search( const int abort_key, const bool again = false,
+                     const std::string & s_ini = "",
+                     const bool toggle = false );
 
 } // end namespace Window_vector
